;*******************************************************************************
;*
;*      Titel:	Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
;*              Tastatur-Emulation
;*              (usb_small_lib_hid_keyboard_firmware_v1_3_a.asm)
;*              
;*	Datum:	11/03/12	Version:	1.3 (07/04/16)
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*      Controller: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u4
;*                  (siehe Zuweisungen weiter unten)
;*      Quarz:      8 oder 16 MHz  (siehe Zuweisungen weiter unten)
;*      
;*      Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
;*     
;*      Eingaenge:   PB0-7 (8 Tasten)
;*      Ausgaenge:   PD6 (LED on board)
;*            
;*      AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
;*
;*      Eingaenge:   PB0-7 (8 Tasten) ev. anpassen
;*      Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
;*      
;*      Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
;*
;*      Eingaenge:   PB0-7 (8 Tasten)
;*      Ausgaenge:   PD6 (LED on board)
;*                  
;*      Informationen zur Funktionsweise: 
;*      
;*      Hauptprogramm zur Nutzung der HID-Bibliothek
;*      Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
;*      Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
;*      abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
;*      USB-Bibliothek.
;*      
;*      Fuer beide Endpunkte steht ein 64-Byte großer Buffer zur Verfuegung. 
;*      
;*      Beim Endpunkt 1 (IN) fragt der PC Daten an.
;*      Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
;*      neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
;*      Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP1_BUF) abgelegt.
;*      
;*      Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
;*      Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
;*      Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie
;*      viele Bytes gesendet wurden.
;*      
;*      Es wird nach 5 Sekunden Wartezeit einmalig der Text "Hallo" gesendet.
;*      Danach werden die 8 Bit (Tasten) von Port B eingelesen (nullaktiv).
;*      Wurde eine Taste betätigt, so wird ein Zeichen gesendet.
;*      
;*      Weitere Dokumentation auf www.weigu.lu/b/usb
;*      
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*      
;*      This program is free software: you can redistribute it and/or modify
;*      it under the terms of the GNU General Public License as published by
;*      the Free Software Foundation, either version 3 of the License, or
;*      (at your option) any later version.
;*
;*      This program is distributed in the hope that it will be useful,
;*      but WITHOUT ANY WARRANTY; without even the implied warranty of
;*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*      GNU General Public License for more details.
;*
;*      You should have received a copy of the GNU General Public License
;*      along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;*      Licence: GPL-3
;*
;*******************************************************************************

;-------------------------------------------------------------------------------
;       Einbinden der controllerspez.Definitionsdatei und Auswahl des Boards
;-------------------------------------------------------------------------------
.NOLIST                         ;List-Output ausschalten

;Hier bitte auswaehlen welches Board benutzt wir!!
;Teensy 2.0: ATmega32u4  16 MHz
;AT90USBKEY: AT90USB1287  8 MHz
;Teensy 1.0: AT90USB162  16 MHz

.EQU TEENSY2    = 1             ;eines der Flags auf 1, die anderen auf 0
.EQU AT90USBKEY = 0
.EQU TEENSY1    = 0

.IF TEENSY2 == 1
.INCLUDE "m32u4def.inc"         ;AVR-Definitionsdatei einbinden
.EQU CRYSTAL = 16
.ENDIF

.IF AT90USBKEY == 1             ;!Achtung es wurde eine Zeit-Bibliothek fuer
.INCLUDE "usb1287def.inc"       ;16MHz verwendet. Die Zeiten verdoppeln sich
.EQU CRYSTAL = 8                ;fuer dieses Board (W5s dauert 10 Sekunden)!
.ENDIF

.IF TEENSY1 == 1
.INCLUDE "usb162def.inc"
.EQU CRYSTAL = 16
.ENDIF

.LIST                           ;List-Output wieder einschalten

;-------------------------------------------------------------------------------
;       Organisation des Datenspeichers (SRAM)
;-------------------------------------------------------------------------------
.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
.ORG    SRAM_START

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Programmspeicher (FLASH)     Programmstart nach RESET ab Adr. 0x0000
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
.ORG    0x0000                  ;Programm beginnt an der FLASH-Adresse 0x0000
RESET1: rjmp    INIT            ;springe nach INIT (ueberspringe ISR Vektoren)

;-------------------------------------------------------------------------------
;       Sprungadressen fuer die Interrupts organisieren (ISR VECTORS)
;-------------------------------------------------------------------------------
        ;Vektortabelle (im Flash-Speicher)
.ORG    USB_GENaddr             ;interner Vektor fuer USB General Interrupt
        rjmp    ISRGEN          ;Springe zur ISR des GENeral Interrupt
.ORG    USB_COMaddr             ;interner Vektor fuer USB Endpoint/Pipe
                                ;COMmunication Interrupt
        rjmp    ISRCOM          ;Springe zur ISR des EP COMmunication Interrupt
.ORG    OC1Aaddr                ;Timer/Counter1 Compare Match A
        rjmp    ISRTC1          ;springe zur ISR Timer1

;-------------------------------------------------------------------------------
;       Initialisierungen und eigene Definitionen
;-------------------------------------------------------------------------------
.ORG    INT_VECTORS_SIZE        ;Platz fuer ISR Vektoren lassen
INIT:
.DEF    Zero  = r15             ;Register 1 wird zum Rechnen benoetigt
        clr     r15             ;und mit Null belegt
.DEF    Tmp1  = r16             ;Register 16 dient als erster Zwischenspeicher
.DEF    Tmp2  = r17             ;Register 17 dient als zweiter Zwischenspeicher
.DEF    Cnt1  = r18             ;Register 18 dient als Zaehler
.DEF    WL    = r24             ;Register 24 und 25 dienen als universelles
.DEF    WH    = r25             ;Doppelregister W und zur Parameteruebergabe
.DEF    W     = r24

;uncomment if not needed
;falls nicht erwünscht, auskommentieren 
;.EQU MOUSE = 1
.EQU KEYBOARD = 1

        ;Stapel initialisieren (fuer Unterprogramme bzw. Interrupts)
        ldi     Tmp1,HIGH(RAMEND)       ;RAMEND (SRAM) ist in der Definitions-
        out     SPH,Tmp1                ;datei festgelegt
        ldi     Tmp1,LOW(RAMEND)
        out     SPL,Tmp1

        ;Im Originalzustand ist das Fusebit CKDIV8 gesetzt, was eine Teilung
        ;durch 8 der Frequenz bewirkt
        ldi     Tmp1,0x80        ;CLKPCE = 1 Wirkung des Fusebit CKDIV
        sts     CLKPR,Tmp1       ;ueberschreiben
        clr     Tmp1             ;CLKPR = 0, Teiler = 1
        sts     CLKPR,Tmp1

        ;Timer 1 initialisieren
        ;Interrupt zwei mal pro Sekunde
        ;Vorteiler 1024, Zaehlschritte 7812-1 ergeben bei 16 MHz 1/2 Sekunde
        clr     Tmp1
        sts     TCCR1A,Tmp1
        ldi     Tmp1,0b00001101 ;WGM12 = 1 Teiler = 1024
        sts     TCCR1B,Tmp1
.IF CRYSTAL == 8
        ldi     Tmp1,HIGH(3905) ;8MHz
        ldi     Tmp2,LOW(3905)  ;16MHz
.ELSE        
        ldi     Tmp1,HIGH(7811)
        ldi     Tmp2,LOW(7811)
.ENDIF        
        sts     OCR1AH,Tmp1     ;Achtung High zuerst!
        sts     OCR1AL,Tmp2
        ldi     Tmp1,0x02       ;OCI1A Interrupt erlauben
        sts     TIMSK1,Tmp1

        ;Eingaenge festlegen (! Anpassen je nach Chip)
        out     DDRB,Zero       ;8 Tasten an PORTB
        ser     Tmp1
        out     PORTB,Tmp1      ;Pull-Ups
        ;Ausgaenge initialisieren
        sbi     DDRD,6          ;PD6 = Ausgang

        ;USB starten (erlaubt auch Interrupts (sei))
        rcall   USBINI          ;befindet sich in der USB-Bibliothek

        rcall   W5s             ;Warte 5 Sekunden
        ;Schreibe "Hallo"
        ldi     r24,0x02        ;Shift
        ldi     r25,0x0B        ;'H'
        rcall   SNDKEY
        ldi     r24,0x00        
        ldi     r25,0x04        ;'a'
        rcall   SNDKEY
        ldi     r24,0x00        
        ldi     r25,0x0F        ;'l'
        rcall   SNDKEY
        ldi     r24,0x00        
        ldi     r25,0x0F        ;'l'
        rcall   SNDKEY
        ldi     r24,0x00        
        ldi     r25,0x12        ;'o'
        rcall   SNDKEY
        ldi     r24,0x00        
        ldi     r25,0x2C        ;Space
        rcall   SNDKEY
        
;-------------------------------------------------------------------------------
;        Hauptprogramm
;-------------------------------------------------------------------------------
; 8 Tasten abfragen      
MAIN:   sbic    PINB,0
        rjmp    M1
        ldi     r24,0x00        
        ldi     r25,0x14        ;'q'
        rcall   SNDKEY
M1:     sbic    PINB,1
        rjmp    M2
        ldi     r24,0x00        
        ldi     r25,0x1A        ;'w'
        rcall   SNDKEY
M2:     sbic    PINB,2
        rjmp    M3
        ldi     r24,0x00        
        ldi     r25,0x08        ;'e'
        rcall   SNDKEY
M3:     sbic    PINB,3
        rjmp    M4
        ldi     r24,0x00        
        ldi     r25,0x15        ;'r'
        rcall   SNDKEY
M4:     sbic    PINB,4
        rjmp    M5
        ldi     r24,0x00        
        ldi     r25,0x17        ;'t'
        rcall   SNDKEY
M5:     sbic    PINB,5
        rjmp    M6
        ldi     r24,0x00        
        ldi     r25,0x1C        ;'z'
        rcall   SNDKEY
M6:     sbic    PINB,6
        rjmp    M7
        ldi     r24,0x00        
        ldi     r25,0x18        ;'u'
        rcall   SNDKEY
M7:     sbic    PINB,7
        rjmp    M8
        ldi     r24,0x00        
        ldi     r25,0x0C        ;'i'
        rcall   SNDKEY
        
M8:     rcall   W100ms

        rjmp    MAIN            ;Endlosschleife

;-------------------------------------------------------------------------------
;       Unterprogramme und Interrupt-Behandlungsroutinen
;-------------------------------------------------------------------------------
;Timer 1 ISR Interrupt jede Sekunde
ISRTC1: push    r16             ;rette r16 (als lokale Variable verwendet)
        in      r16,SREG        ;SREG-Register (Flags) retten
        push    r16

        sbi     PIND,6          ;Toggle LED

        pop     r16             ;SREG-Register (Flags) wiederherstellen
        out     SREG,r16        
        pop     r16             ;r16 wiederherstellen
        reti            

;Unterprogramm zum Senden eines Zeichens 
;Parameter:
;r24: byte for modifier key, Bit 7-0:
;R GUI, R ALT, R SHIFT, R CTRL,L GUI, L ALT, L SHIFT, L CTRL
;r25: scan code from key 
SNDKEY: push    r16
        ldi     r16,8           ;8 bytes has to be 8 Bytes for windows!!
        sts     EP1_CNT,r16
        cli
        sts     EP1_BUF+0,r24     ;modifier byte
        sts     EP1_BUF+1,Zero  ;reserved byte = 0
        sts     EP1_BUF+2,r25   ;scan code (up to 6 keys)
        sts     EP1_BUF+3,Zero
        sts     EP1_BUF+4,Zero
        sts     EP1_BUF+5,Zero
        sts     EP1_BUF+6,Zero
        sts     EP1_BUF+7,Zero
        sei
        ldi     r16,1           ;EP1 Flag setzen weil Daten vorhanden
        sts     EP1_FLAG,r16
        rcall   W5ms            ;higher than bInterval in Endpoint Descriptor!
        ldi     r16,8           ;8 bytes
        sts     EP1_CNT,r16
        cli
        sts     EP1_BUF,Zero    ;modifier byte
        sts     EP1_BUF+1,Zero  ;reserved byte = 0
        sts     EP1_BUF+2,Zero  ;scan code (up to 6 keys)
        sts     EP1_BUF+3,Zero
        sts     EP1_BUF+4,Zero
        sts     EP1_BUF+5,Zero
        sts     EP1_BUF+6,Zero
        sts     EP1_BUF+7,Zero
        sei
        ldi     r16,1           ;EP1 Flag setzen weil Daten vorhanden
        sts     EP1_FLAG,r16
        rcall   W5ms            ;higher than bInterval in Endpoint Descriptor!
        pop     r16
        ret  
        
.INCLUDE "lib/SR_USB_HID_V1_3.asm" ;USB Bibliothek
.IF AT_90USB1287 == 1    
.INCLUDE "lib/SR_TIME_8M.asm"  ;Zeit-Bibliothek fuer 8MHz Quarz!
.ELSE
.INCLUDE "lib/SR_TIME_16M.asm" ;Zeit-Bibliothek fuer 16MHz Quarz!
.ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EXIT                           ;Ende des Quelltextes
