/*! \file usb_small_lib_hid_firmware_v1_2_c.c
********************************************************************************
 main source for using the small usb library using hid class
 Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
 
 www.weigu.lu/b/usb

 (inspired by the lib from S. Salewski (http://http://www.ssalewski.de)
 
 \author weigu www.weigu.lu

        Controller: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
                   (siehe Zuweisungen makefile)
        Quarz:     8 oder 16 MHz  (siehe Zuweisungen makefile)

        In der Makefile muss der Chip und die Quarzfrequenz für das
        entsprechende Board festgelegt werden!

        Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):

        Eingaenge:   ADC0 (PF0) analog
        Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS

        AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):

        Eingaenge:   Temperatursensor an ADC0 (PF0) analog
        Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)

        Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):

        Eingaenge:   PC4
        Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS

        Informationen zur Funktionsweise:

        Hauptprogramm zur Nutzung der HID-Bibliothek
        Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
        Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
        abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
        USB-Bibliothek.
        
        Fuer beide Endpunkte steht ein 64-Byte grosser Buffer zur Verfuegung. 

        Beim Endpunkt 1 (IN) fragt der PC Daten an.
        Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
        neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
        Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP1_BUF) abgelegt.
        
        Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
        Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
        Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie 
        viele Bytes gesendet wurden.
        
        Ueber Endpunkt 1 wird dem PC der Wert des AD-Wandlers (ADC0, 2 Byte))
        uebermittelt.
        Endpunkt 2 liefert 1 Byte, welche die Information enthaelt, welche LEDs
        eingeschaltet sind. Bit 6 schaltet das Blinken ein- bzw. aus (Blinkflag)

******************************************************************************/


/* Copyright (c) 2013   Guy WEILER         weigu[at]weigu[dot]lu

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Licence: GPL-3        */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "lib/usb_srs_hid_v1_2.h"

/*! \brief  Timer1 Compare Match A Interrupt
    Toggelt LED an PD6 */
ISR (TIMER1_COMPA_vect)
{
  SBI (PIND,6);         // Toggelt bei neuen Chips das Pin!  
}

/*! \brief Hauptprogramm
 */
int main(void)
{
  // Wirkung des Fusebit CKDIV ueberschreiben
  CLKPR = (1<<CLKPCE);  
  CLKPR = 0;
  // Timer1 initialisieren, Interrupt ein mal pro Sekunde
  // Vorteiler 1024, Zaehlschritte 7811-1 ergeben bei 16MHz 1 Sekunde
  TCCR1A = 0;
  TCCR1B = 0x0D;                // WGM12 = 1 Teiler = 1024
#if F_CPU == 8000000
  OCR1AH = high(3905);          // Achtung High zuerst!
  OCR1AL = low(3905);
#else
  OCR1AH = high(7811);          // Achtung High zuerst!
  OCR1AL = low(7811);
#endif
  TIMSK1 |= (1 << OCIE1A);      // OCI1A Interrupt erlauben
  // AD-Wandler initialisieren
  // Der AT90USB162 bzw. mega32u2 besitzt keinen A/D-Wandler
#if defined(__AVR_AT90USB162__) || defined(__AVR_ATmega32U2__) 
  DDRC &= ~(1<<DDC4);           // Es wird deshalb PC4 als Eingang eingelesen
  PORTC |= (1<<PC4);            // Pull-Up
#else                           // ADC initialisieren
  ADMUX = 0b01000000;   // REFS = 01 AVCC, ADLAR = 0 rechtsbuendig, ADC0
  ADCSRA = 0b10000111;  // ADEN = 1 (ADC einschalten), ADPS = 111
#endif                  // (16MHz/128 = 125kHz), andere Bits per default = 0
  // Ausgänge festlegen
#if defined(__AVR_AT90USB1287__) // Der AT90USBKEY hat 4 LEDs on board
  DDRD |= ((1<<DDD4) | (1<<DDD5) | (1<<DDD6) | (1<<DDD7)); // PD4-7 = Ausgang
#else                         
  DDRD |= (1<<DDD6);    // PD6 = Ausgang Teensy on board
  DDRB |= ((1<<DDB3) | (1<<DDB2) | (1<<DDB1) | (1<<DDB0)); // PB0-3 = Ausgang
#endif
  
  // USB starten (erlaubt auch Interrupts (sei))
  usb_init_device();    // Unterprogramm der USB-Bibliothek
  blinkflag = 1;
 
  while (1)
  {
    Ep1_cnt = 2;        // Es werden 2 Werte übergeben
#if defined(__AVR_AT90USB162__) || defined(__AVR_ATmega32U2__) 
    cli();
    if ((PINC & (1<<PC4)) == 0) // AT90USB162 besitzt keinen A/D-Wandler
    {                           // Es wird deshalb PC4 als Eingang eingelesen
      Ep1_buf[0] = 0;
      Ep1_buf[1] = 0;
    }  
    else
    {        
      Ep1_buf[0] = low(1023);
      Ep1_buf[1] = high(1023);
    }
    sei();
#else
    ADCSRA |= (1<<ADSC);         // ADC Wandlung starten (ADSC = 1)
    while ((ADCSRA &(1<<ADSC))); //und warten bis Wandlung fertig (ADSC = 0)
    cli();
    Ep1_buf[0] = ADCL;
    Ep1_buf[1] = ADCH;
    sei();
#endif
    Ep1_flag = 1;               // EP1 Flag setzen weil Daten vorhanden
    
    if (Ep2_flag == 1)
    {
      blinkflag = (Ep2_buf[0] & 0x40);  // Blinkflag ausmaskieren
#if defined(__AVR_AT90USB1287__)        // Der AT90USBKEY hat 4 LEDs on board
      PORTD |= __builtin_avr_swap((Ep2_buf[0] & 0x0F)); // LEDs mit Maskierung setzen
      PORTD &= __builtin_avr_swap((Ep2_buf[0] | 0xF0)); // LEDs mit Maskierung loeschen
#else
      PORTB |= (Ep2_buf[0] & 0x0F); // LEDs mit Maskierung setzen
      PORTB &= (Ep2_buf[0] | 0xF0); // LEDs mit Maskierung loeschen
#endif
    Ep2_flag = 0; //Flag zuruecksetzen
    }
    if (blinkflag) TIMSK1 |= (1 << OCIE1A);     // OCI1A Interrupt erlauben
    else TIMSK1 &= ~(1 << OCIE1A);              // OCI1A Interrupt sperren
  }
}