/*! \file usb_small_lib_hid_mouse_firmware_v1_2_c.c
********************************************************************************
 main source for using the small usb library using hid class
 HID mouse
 Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
 Maus-Emulation
 
 www.weigu.lu/b/usb

 (inspired by the lib from S. Salewski (http://http://www.ssalewski.de)
 
 \author weigu www.weigu.lu

        Controller: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
                   (siehe Zuweisungen makefile)
        Quarz:     8 oder 16 MHz  (siehe Zuweisungen makefile)

        In der Makefile muss der Chip und die Quarzfrequenz für das
        entsprechende Board festgelegt werden!

        Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
     
        Eingaenge:   PB7 (Joysticktaste), PB0 (Joystick Links)
                     PB1 (Joystick Rauf), PB2 (Joystick Rechts)
                     PB3 (Joystick Runter)
        Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS
       
        AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
  
        Eingaenge:   PB5 (Joysticktaste), PB6 (Joystick Links)
                     PB7 (Joystick Rauf), PE4 (Joystick Rechts)
                     PE5 (Joystick Runter)
        Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
        
        Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
 
        Eingaenge:   PB7 (Joysticktaste), PB0 (Joystick Links)
                     PB1 (Joystick Rauf), PB2 (Joystick Rechts)
                     PB3 (Joystick Runter)
        Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS

        Informationen zur Funktionsweise:

        Hauptprogramm zur Nutzung der HID-Bibliothek
        Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
        Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
        abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
        USB-Bibliothek.
        
        Fuer beide Endpunkte steht ein 64-Byte großer Buffer zur Verfuegung. 

        Beim Endpunkt 1 (IN) fragt der PC Daten an.
        Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
        neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
        Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP1_BUF) abgelegt.
        
        Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
        Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
        Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie 
        viele Bytes gesendet wurden.
        
        Das Freischalten der Maus-Emulation erfolgt in der Headerdatei
        lib/usb_srs_hid_v1_2.h
      
        Weitere Dokumentation auf www.weigu.lu/b/usb

******************************************************************************/


/* Copyright (c) 2013   Guy WEILER         weigu[at]weigu[dot]lu

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Licence: GPL-3        */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "lib/usb_srs_hid_v1_2.h"

/*! \brief  Timer1 Compare Match A Interrupt
    Toggelt LED an PD6 */
ISR (TIMER1_COMPA_vect)
{
  SBI (PIND,6);                 // Toggelt bei neuen Chips das Pin!  
}

/*! \brief Hauptprogramm
 */
int main(void)
{
  // Wirkung des Fusebit CKDIV ueberschreiben
  CLKPR = (1<<CLKPCE);  
  CLKPR = 0;
  // Timer1 initialisieren, Interrupt ein mal pro Sekunde
  // Vorteiler 1024, Zaehlschritte 7811-1 ergeben bei 16MHz 1 Sekunde
  TCCR1A = 0;
  TCCR1B = 0x0D;                // WGM12 = 1 Teiler = 1024
#if F_CPU == 8000000
  OCR1AH = high(3905);          // Achtung High zuerst!
  OCR1AL = low(3905);
#else
  OCR1AH = high(7811);          // Achtung High zuerst!
  OCR1AL = low(7811);
#endif
  TIMSK1 |= (1 << OCIE1A);      // CI1A Interrupt erlauben
  
  // Ein und Ausgänge
#if defined(__AVR_AT90USB1287__) 
  // PD4-7 = Ausgang
  DDRD |= ((1<<DDD4) | (1<<DDD5) | (1<<DDD6) | (1<<DDD7));
  // Eingänge Joystick PB5-7, PE4-5
  DDRB &= ~((1<<DDB5) | (1<<DDB6) | (1<<DDB7)); // Joystick Taste Links Rauf
  DDRE &= ~((1<<DDE4) | (1<<DDE4));             // Joystick Rechts Runter
  PORTB |= ((1<<PB5) | (1<<PB6) | (1<<PB7));    // Pull-Ups
  PORTE |= ((1<<PE4) | (1<<PE5));               // Pull-Ups
#define Butt1 (PINB & (1 << PB5))
#define X_left (PINB & (1 << PB6))
#define Y_left (PINB & (1 << PB7))
#define X_right (PINE & (1 << PE4))
#define Y_right (PINE & (1 << PE5))           
  
#else
  // PD6 = Ausgang
  DDRD |= (1<<DDD6);
  // Eingänge Joystick PB0-3 (Links Rauf Rechts Runter), PB7 Taste
  DDRB &= ~((1<<DDB0) | (1<<DDB1) | (1<<DDB2) | (1<<DDB3) | (1<<DDB7));
  PORTB |= ((1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3) | (1<<PB7)); // Pull-Ups
#define Butt1 (PINB & (1 << PB7))
#define X_left (PINB & (1 << PB0))
#define Y_left (PINB & (1 << PB1))
#define X_right (PINB & (1 << PB2))
#define Y_right (PINB & (1 << PB3))
  
#endif  
  
  // USB starten (erlaubt auch Interrupts (sei))
  usb_init_device();    // Unterprogramm der USB-Bibliothek
 
  while (1)
  {
    Ep1_cnt = 4;                        // Maus liefert 4 Byte
    // Linke Maustaste
    if (Butt1 == 0) 
      Ep1_buf[0] = 1;                   // Falls Button gedrueckt(nullak.)
    else                                // dann setze Butt1 (Linke Maustaste)
      Ep1_buf[0] = 0;                     
   // X-Koordinate
   if (X_right == 0)
     Ep1_buf[1] = 1;                    // Falls Rechts gedrueckt X = X+1
   else if (X_left == 0) 
     Ep1_buf[1] = 0xFF;                 // Falls Links X = X-1 (Zweierkompl.)
   else
     Ep1_buf[1] = 0;
   // Y-Koordinate
   if (Y_right == 0) 
     Ep1_buf[2] = 1;                    // Falls Runter gedrueckt Y = Y+1
   else if (Y_left == 0) 
     Ep1_buf[2] = 0xFF;                 // Falls Rauf Y = Y -1
   else
     Ep1_buf[2] = 0;
   // Scrollrad
   Ep1_buf[3] = 0;
   Ep1_flag = 1;                        // Flag dass Daten vorhanden setzen    
  }
}